﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.IO;
using Microsoft.Win32;

using wclCommon;
using wclCommunication;
using wclIrDA;

namespace FtpClient
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private wclObexFtpClient FFtp;
        private wclIrDAReceiver wclIrDAReceiver;
        private wclIrDAClient wclIrDAClient;

        internal class DeviceItem
        {
            public String Address { get; set; }
            public String Name { get; set; }
        };

        internal class FileItem
        {
            public String ObjectName { get; set; }
            public String Type { get; set; }
            public String Size { get; set; }
            public String Permission { get; set; }
            public String Modified { get; set; }
            public String Created { get; set; }
            public String Accessed { get; set; }
            public String Description { get; set; }
        };

        public MainWindow()
        {
            InitializeComponent();
        }

        private void fmMain_Loaded(object sender, RoutedEventArgs e)
        {
            wclIrDAReceiver = new wclIrDAReceiver();

            wclIrDAClient = new wclIrDAClient();
            wclIrDAClient.OnDisconnect += new wclClientConnectionDisconnectEvent(wclIrDAClient_OnDisconnect);
            wclIrDAClient.OnConnect += new wclClientConnectionConnectEvent(wclIrDAClient_OnConnect);
            wclIrDAClient.OnDestroyProcessor += new wclClientConnectionProcessorEvent(wclIrDAClient_OnDestroyProcessor);
            wclIrDAClient.OnCreateProcessor += new wclClientConnectionProcessorEvent(wclIrDAClient_OnCreateProcessor);

            // We use ObjectPush Profile so must set the profile's UUID.
            wclIrDAClient.Service = "OBEX";
            wclIrDAClient.Mode = wclIrDAConnectionMode.cmIrComm3Wire; // We ned this for OBEX communication.

            FFtp = null;
        }

        void wclIrDAClient_OnCreateProcessor(object Sender, wclClientDataConnection Connection)
        {
            // here we must create the data processor for the connection. In this demo
            // we use OPPClient.
            FFtp = new wclObexFtpClient(Connection);
            FFtp.OnConnect += new wclObexOperationResultEvent(FFtp_OnConnect);
            FFtp.OnDisconnect += new wclObexOperationResultEvent(FFtp_OnDisconnect);
            FFtp.OnGetComplete += new wclObexObjectOperationResultEvent(FFtp_OnGetComplete);
            FFtp.OnPutComplete += new wclObexObjectOperationResultEvent(FFtp_OnPutComplete);
            FFtp.OnProgress += new wclObexOperationProgressEvent(FFtp_OnProgress);
            FFtp.OnDirComplete += new wclObexFtpDirCompleteEvent(FFtp_OnDirComplete);
            FFtp.OnChangeDirComplete += new wclObexOperationResultEvent(FFtp_OnChangeDirComplete);
            FFtp.OnDeleteComplete += new wclObexOperationResultEvent(FFtp_OnDeleteComplete);
            FFtp.OnMakeDirComplete += new wclObexOperationResultEvent(FFtp_OnMakeDirComplete);
        }

        void FFtp_OnMakeDirComplete(object Sender, int Error, string Description)
        {
            lbLog.Items.Add("Make dir completed with result: 0x" + Error.ToString("X8") + ". Description: " + Description);

            ProgressBar.Value = 0;
            ProgressBar.Maximum = 0;

            lvFiles.Items.Clear();
            FFtp.Dir();
        }

        void FFtp_OnDeleteComplete(object Sender, int Error, string Description)
        {
            lbLog.Items.Add("Delete completed with result: 0x" + Error.ToString("X8") + ". Description: " + Description);

            ProgressBar.Value = 0;
            ProgressBar.Maximum = 0;

            lvFiles.Items.Clear();
            FFtp.Dir();
        }

        void FFtp_OnChangeDirComplete(object Sender, int Error, string Description)
        {
            lbLog.Items.Add("Change dir completed with result: 0x" + Error.ToString("X8") + ". Description: " + Description);

            ProgressBar.Value = 0;
            ProgressBar.Maximum = 0;

            lvFiles.Items.Clear();
            FFtp.Dir();
        }

        void FFtp_OnDirComplete(object Sender, int Error, string Description, wclObexFileObject[] Dirs)
        {
            lbLog.Items.Add("Dir completed with result: 0x" + Error.ToString("X8") + ". Description: " + Description);

            ProgressBar.Value = 0;
            ProgressBar.Maximum = 0;

            if (Dirs != null)
            {
                foreach (wclObexFileObject Obj in Dirs)
                {
                    String ObjType;
                    if (Obj.IsDirectory)
                        ObjType = "Folder";
                    else
                        ObjType = "File";

                    String Perm = "";
                    if ((Obj.Permissions & wclObexFilePermission.opRead) != 0)
                        Perm = Perm + "R";
                    if ((Obj.Permissions & wclObexFilePermission.opWrite) != 0)
                        Perm = Perm + "W";
                    if ((Obj.Permissions & wclObexFilePermission.opDelete) != 0)
                        Perm = Perm + "D";
                    String modified = "";
                    if (!Obj.Modified.Equals(DateTime.MinValue))
                        modified = Obj.Modified.ToString();
                    String created = "";
                    if (!Obj.Created.Equals(DateTime.MinValue))
                        created = Obj.Created.ToString();
                    String accessed = "";
                    if (!Obj.Accessed.Equals(DateTime.MinValue))
                        accessed = Obj.Accessed.ToString();

                    lvFiles.Items.Add(new FileItem
                    {
                        ObjectName = Obj.Name,
                        Type = ObjType,
                        Size = Obj.Size.ToString(),
                        Permission = Perm,
                        Modified = modified,
                        Created = created,
                        Accessed = accessed,
                        Description = Obj.Description
                    });
                }
            }
        }

        void FFtp_OnGetComplete(object Sender, int Error, string Description, System.IO.Stream Stream)
        {
            lbLog.Items.Add("File receiving completed with result: 0x" + Error.ToString("X8") + ". Description: " + Description);

            ProgressBar.Value = 0;
            ProgressBar.Maximum = 0;

            // We must destroy the stream object here.
            Stream.Close();
            Stream.Dispose();
        }

        void FFtp_OnProgress(object Sender, uint Size, uint Position)
        {
            ProgressBar.Maximum = (int)Size;
            ProgressBar.Value = (int)Position;
        }

        void FFtp_OnPutComplete(object Sender, int Error, string Description, System.IO.Stream Stream)
        {
            lbLog.Items.Add("File sending completed with result: 0x" + Error.ToString("X8") + ". Description: " + Description);

            ProgressBar.Value = 0;
            ProgressBar.Maximum = 0;

            // We must destroy the stream object here.
            Stream.Close();
            Stream.Dispose();
        }

        void FFtp_OnDisconnect(object Sender, int Error, string Description)
        {
            lbLog.Items.Add("OBEX session closed with result: 0x" + Error.ToString("X8") + ". Description: " + Description);
        }

        void FFtp_OnConnect(object Sender, int Error, string Description)
        {
            lbLog.Items.Add("OBEX session opened with result: 0x" + Error.ToString("X8") + ". Description: " + Description);
        }

        void wclIrDAClient_OnDestroyProcessor(object Sender, wclClientDataConnection Connection)
        {
            // Do we have any data processor created?
            if (FFtp != null)
            {
                // Make sure it is our connection is goind to destroy.
                if (Connection.Processor == FFtp)
                {
                    // Ok, destroy the data processor here.
                    FFtp.Dispose();
                    FFtp = null;
                }
            }
        }

        void wclIrDAClient_OnConnect(object Sender, int Error)
        {
            lbLog.Items.Add("Connect. Operation result: 0x" + Error.ToString("X8"));
        }

        void wclIrDAClient_OnDisconnect(object Sender, int Reason)
        {
            lbLog.Items.Add("Disconnected with reason: 0x" + Reason.ToString("X8"));
        }

        private void btClear_Click(object sender, RoutedEventArgs e)
        {
            lbLog.Items.Clear();
        }

        private void fmMain_Closed(object sender, EventArgs e)
        {
            wclIrDAClient.Disconnect();
        }

        private void btDiscover_Click(object sender, RoutedEventArgs e)
        {
            lvDevices.Items.Clear();

            wclIrDADevice[] Devices = null;
            Int32 Res = wclIrDAReceiver.Discover(out Devices);
            if (Res != wclErrors.WCL_E_SUCCESS)
                MessageBox.Show("Failed to start discovering: 0x" + Res.ToString("X8"));
            else
            {
                if (Devices != null)
                {
                    foreach (wclIrDADevice Device in Devices)
                    {
                        lvDevices.Items.Add(new DeviceItem
                        {
                            Address = Device.Address.ToString("X8"),
                            Name = Device.Name
                        });
                    }
                }
            }
        }

        private void btDisconnect_Click(object sender, RoutedEventArgs e)
        {
            Int32 Res = wclIrDAClient.Disconnect();
            if (Res != wclErrors.WCL_E_SUCCESS)
                MessageBox.Show("Disconnect failed: 0x" + Res.ToString("X8"));
        }

        private void btCloseSession_Click(object sender, RoutedEventArgs e)
        {
            if (FFtp == null)
                MessageBox.Show("Not connected");
            else
            {
                Int32 Res = FFtp.Disconnect("");
                if (Res != wclErrors.WCL_E_SUCCESS)
                    MessageBox.Show("Close session failed: 0x" + Res.ToString("X8"));
            }
        }

        private void btOpenSession_Click(object sender, RoutedEventArgs e)
        {
            if (FFtp == null)
                MessageBox.Show("Not connected");
            else
            {
                Int32 Res = FFtp.Connect();
                if (Res != wclErrors.WCL_E_SUCCESS)
                    MessageBox.Show("Open session failed: 0x" + Res.ToString("X8"));
            }
        }

        private void btConnect_Click(object sender, RoutedEventArgs e)
        {
            if (wclIrDAClient.State != wclClientState.csDisconnected)
                MessageBox.Show("Client is connected");
            else
            {
                if (lvDevices.SelectedItems.Count == 0)
                    MessageBox.Show("Select device");
                else
                {
                    UInt32 Addr = Convert.ToUInt32(((DeviceItem)lvDevices.SelectedItems[0]).Address, 16);
                    wclIrDAClient.Address = Addr;
                    Int32 Res = wclIrDAClient.Connect();
                    if (Res != wclErrors.WCL_E_SUCCESS)
                        MessageBox.Show("Failed to connect: 0x" + Res.ToString("X8"));
                }
            }
        }

        private void btSelectFile_Click(object sender, RoutedEventArgs e)
        {
            OpenFileDialog openFileDialog = new OpenFileDialog();
            if (openFileDialog.ShowDialog() == true)
                edFileName.Text = openFileDialog.FileName;
        }

        private void btAbort_Click(object sender, RoutedEventArgs e)
        {
            if (FFtp == null)
                MessageBox.Show("Not connected");
            else
            {
                Int32 Res = FFtp.Abort("");
                if (Res != wclErrors.WCL_E_SUCCESS)
                    MessageBox.Show("Abort failed: 0x" + Res.ToString("X8"));
            }
        }

        private void btSend_Click(object sender, RoutedEventArgs e)
        {
            if (FFtp == null)
                MessageBox.Show("Not connected");
            else
            {
                if (edFileName.Text == "")
                    MessageBox.Show("Select file");
                else
                {
                    FileStream Stream = new FileStream(edFileName.Text, FileMode.Open);
                    Int32 Res = FFtp.Put(System.IO.Path.GetFileName(edFileName.Text), "", Stream);
                    if (Res != wclErrors.WCL_E_SUCCESS)
                    {
                        MessageBox.Show("Send file failed: 0x" + Res.ToString("X8"));
                        Stream.Close();
                        Stream.Dispose();
                    }
                }
            }
        }

        private void btDir_Click(object sender, RoutedEventArgs e)
        {
            if (FFtp == null)
                MessageBox.Show("Not connected");
            else
            {
                Int32 Res = FFtp.Dir();
                if (Res != wclErrors.WCL_E_SUCCESS)
                    MessageBox.Show("Dir failed: 0x" + Res.ToString("X8"));
                else
                    lvFiles.Items.Clear();
            }
        }

        private void lvFiles_MouseDoubleClick(object sender, MouseButtonEventArgs e)
        {
            if (lvFiles.SelectedItems.Count > 0)
            {
                if (FFtp == null)
                    MessageBox.Show("Not connected");
                else
                {
                    if (((FileItem)lvFiles.SelectedItems[0]).Type == "Folder")
                    {
                        Int32 Res = FFtp.ChangeDir(((FileItem)lvFiles.SelectedItems[0]).ObjectName);
                        if (Res != wclErrors.WCL_E_SUCCESS)
                            MessageBox.Show("Change path failed 0x" + Res.ToString("X8"));
                    }
                    else
                    {
                        FileStream Stream = new FileStream(".\\" + ((FileItem)lvFiles.SelectedItems[0]).ObjectName,
                            FileMode.OpenOrCreate, FileAccess.ReadWrite);
                        Int32 Res = FFtp.Get(((FileItem)lvFiles.SelectedItems[0]).ObjectName, Stream);
                        if (Res != wclErrors.WCL_E_SUCCESS)
                        {
                            MessageBox.Show("Get failed 0x" + Res.ToString("X8"));
                            Stream.Close();
                            Stream.Dispose();
                        }
                    }
                }
            }
        }

        private void btRoot_Click(object sender, RoutedEventArgs e)
        {
            if (FFtp == null)
                MessageBox.Show("Not connected");
            else
            {
                Int32 Res = FFtp.ChangeDir("");
                if (Res != wclErrors.WCL_E_SUCCESS)
                    MessageBox.Show("Root failed 0x" + Res.ToString("X8"));
            }
        }

        private void btDelete_Click(object sender, RoutedEventArgs e)
        {
            if (lvFiles.SelectedItems.Count > 0)
            {
                if (FFtp == null)
                    MessageBox.Show("Not connected");
                else
                {
                    Int32 Res = FFtp.Delete(((FileItem)lvFiles.SelectedItems[0]).ObjectName);
                    if (Res != wclErrors.WCL_E_SUCCESS)
                        MessageBox.Show("Delete failed 0x" + Res.ToString("X8"));
                }
            }
        }

        private void btMkDir_Click(object sender, RoutedEventArgs e)
        {
            if (FFtp == null)
                MessageBox.Show("Not connected");
            else
            {
                Int32 Res = FFtp.MkDir(edNewDirName.Text);
                if (Res != wclErrors.WCL_E_SUCCESS)
                    MessageBox.Show("Make sire failed 0x" + Res.ToString("X8"));
            }
        }
    }
}
