﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using wclIrDA;
using wclCommunication;
using wclCommon;

namespace IrDAClient
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private wclIrDAClient wclIrDAClient;
        private wclIrDAReceiver wclIrDAReceiver;

        internal class DeviceItem
        {
            public String Address { get; set; }
            public String Name { get; set; }
        };

        public MainWindow()
        {
            InitializeComponent();
        }

        private void fmMain_Loaded(object sender, RoutedEventArgs e)
        {
            wclIrDAClient = new wclIrDAClient();
            wclIrDAClient.OnConnect += new wclClientConnectionConnectEvent(wclIrDAClient_OnConnect);
            wclIrDAClient.OnData += new wclConnectionDataEvent(wclIrDAClient_OnData);
            wclIrDAClient.OnDisconnect += new wclClientConnectionDisconnectEvent(wclIrDAClient_OnDisconnect);

            wclIrDAReceiver = new wclIrDAReceiver();
        }

        void wclIrDAClient_OnConnect(object Sender, int Error)
        {
            if (Error == wclErrors.WCL_E_SUCCESS)
            {
                lbEvents.Items.Add("Connected");

                // Try to get read and write buffers size.
                UInt32 Size = 0;
                Int32 Res = wclIrDAClient.GetReadBufferSize(out Size);
                if (Res != wclErrors.WCL_E_SUCCESS)
                    lbEvents.Items.Add("Get read buffer error: 0x" + Res.ToString("X8"));
                else
                    edReadBuffer.Text = Size.ToString();

                Size = 0;
                Res = wclIrDAClient.GetWriteBufferSize(out Size);
                if (Res != wclErrors.WCL_E_SUCCESS)
                    lbEvents.Items.Add("Get write buffer error: 0x" + Res.ToString("X8"));
                else
                    edWriteBuffer.Text = Size.ToString();
            }
            else
                lbEvents.Items.Add("Connect error: 0x" + Error.ToString("X8"));
        }

        void wclIrDAClient_OnData(object Sender, byte[] Data)
        {
            if (Data != null && Data.Length > 0)
            {
                String Str = Encoding.ASCII.GetString(Data);

                lbEvents.Items.Add("Received: " + Str);
            }
        }

        void wclIrDAClient_OnDisconnect(object Sender, int Reason)
        {
            lbEvents.Items.Add("Disconnected: 0x" + Reason.ToString("X8"));

            edReadBuffer.Text = "0";
            edWriteBuffer.Text = "0";
        }

        private void btDiscover_Click(object sender, RoutedEventArgs e)
        {
            lvDevices.Items.Clear();

            wclIrDADevice[] Devices = null;
            Int32 Res = wclIrDAReceiver.Discover(out Devices);
            try
            {
                if (Res != wclErrors.WCL_E_SUCCESS)
                    MessageBox.Show("Error: 0x" + Res.ToString("X8"));
                else
                    if (Devices == null || Devices.Length == 0)
                        MessageBox.Show("No devices were found.");
                    else
                        for (int i = 0; i < Devices.Length; i++)
                        {
                            lvDevices.Items.Add(new DeviceItem
                            {
                                Address = Devices[i].Address.ToString("X8"),
                                Name = Devices[i].Name
                            });
                        }
            }
            finally
            {
                Devices = null;
            }
        }

        private void btClear_Click(object sender, RoutedEventArgs e)
        {
            lbEvents.Items.Clear();
        }

        private void btDisconnect_Click(object sender, RoutedEventArgs e)
        {
            Int32 Res = wclIrDAClient.Disconnect();
            if (Res != wclErrors.WCL_E_SUCCESS)
                MessageBox.Show("Error: 0x" + Res.ToString("X8"));
        }

        private void btConnect_Click(object sender, RoutedEventArgs e)
        {
            if (lvDevices.SelectedItems.Count == 0)
                MessageBox.Show("Select device");
            else
            {
                DeviceItem Item = (DeviceItem)lvDevices.SelectedItems[0];
                wclIrDAClient.Address = Convert.ToUInt32(Item.Address, 16);
                wclIrDAClient.Timeout = Convert.ToUInt32(edTimeout.Text);

                Int32 Res = wclIrDAClient.Connect();
                if (Res != wclErrors.WCL_E_SUCCESS)
                    MessageBox.Show("Error: 0x" + Res.ToString("X8"));
            }
        }

        private void btSend_Click(object sender, RoutedEventArgs e)
        {
            Byte[] Ansi = Encoding.ASCII.GetBytes(edText.Text);
            UInt32 Sent = 0;
            Int32 Res = wclIrDAClient.Write(Ansi, out Sent);
            if (Res != wclErrors.WCL_E_SUCCESS)
                MessageBox.Show("Error: 0x" + Res.ToString("X8"));
            else
                MessageBox.Show("Sent: " + Sent.ToString() + " from " + Ansi.Length.ToString());
        }

        private void fmMain_Closed(object sender, EventArgs e)
        {
            wclIrDAClient.Disconnect();
            wclIrDAClient = null;
            wclIrDAReceiver = null;
        }

        private void btSetBuffers_Click(object sender, RoutedEventArgs e)
        {
            // Try to set read buffer size first.
            UInt32 Size = Convert.ToUInt32(edReadBuffer.Text);
            Int32 Res = wclIrDAClient.SetReadBufferSize(Size);
            if (Res == wclErrors.WCL_E_SUCCESS)
                lbEvents.Items.Add("Read buffer size changed.");
            else
                lbEvents.Items.Add("Set read buffer size error: 0x" + Res.ToString("X8"));

            // Now write buffer.
            Size = Convert.ToUInt32(edWriteBuffer.Text);
            Res = wclIrDAClient.SetWriteBufferSize(Size);
            if (Res == wclErrors.WCL_E_SUCCESS)
                lbEvents.Items.Add("Write buffer size changed.");
            else
                lbEvents.Items.Add("Set write buffer size error: 0x" + Res.ToString("X8"));
        }
    }
}
