﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using wclIrDA;
using wclCommon;
using wclCommunication;

namespace IrDAServer
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private wclIrDAServer wclIrDAServer;

        internal class DeviceItem
        {
            public String Address { get; set; }
            public String Name { get; set; }
            internal wclIrDAServerClientConnection Client { get; set; }
        };

        public MainWindow()
        {
            InitializeComponent();
        }

        private void fmMain_Loaded(object sender, RoutedEventArgs e)
        {
            wclIrDAServer = new wclIrDAServer();

            wclIrDAServer.OnClosed += new wclClientConnectionDisconnectEvent(wclIrDAServer_OnClosed);
            wclIrDAServer.OnListen += new EventHandler(wclIrDAServer_OnListen);
            wclIrDAServer.OnConnect += new wclIrDAServerConnectEvent(wclIrDAServer_OnConnect);
            wclIrDAServer.OnDisconnect += new wclIrDAServerDisconnectEvent(wclIrDAServer_OnDisconnect);
            wclIrDAServer.OnData += new wclIrDAServerDataEvent(wclIrDAServer_OnData);  

            cbIasValueType.SelectedIndex = 0;
        }

        void wclIrDAServer_OnClosed(object Sender, int Reason)
        {
            lbEvents.Items.Add("Server closed: 0x" + Reason.ToString("X8"));

            lvClients.Items.Clear();

            edReadBuffer.Text = "";
            edWriteBuffer.Text = "";
        }

        void wclIrDAServer_OnListen(object sender, EventArgs e)
        {
            lbEvents.Items.Add("Server listening");
        }

        void wclIrDAServer_OnConnect(object Sender, wclIrDAServerClientConnection client, int Error)
        {
            lbEvents.Items.Add("Client connected: " + client.Address.ToString("X8") +
                " error: 0x" + Error.ToString("X8"));

            if (Error == wclErrors.WCL_E_SUCCESS)
            {
                lvClients.Items.Add(new DeviceItem {
                    Address = client.Address.ToString("X8"),
                    Name = client.Name,
                    Client = client
                });
            }
        }

        void wclIrDAServer_OnDisconnect(object Sender, wclIrDAServerClientConnection Client, int Reason)
        {
            lbEvents.Items.Add("Client disconnected: " + Client.Address.ToString("X8") +
                " reason: 0x" + Reason.ToString("X8"));

            for (Int32 i = 0; i < lvClients.Items.Count; i++)
                if (((DeviceItem)lvClients.Items[i]).Client == Client)
                {
                    lvClients.Items.RemoveAt(i);
                    break;
                }
        }

        void wclIrDAServer_OnData(object Sender, wclIrDAServerClientConnection Client, byte[] Data)
        {
            if (Data != null && Data.Length > 0)
            {
                String Str = Encoding.ASCII.GetString(Data);

                lbEvents.Items.Add("Received: " + Str + " from " + Client.Address.ToString("X8"));
            }
        }

        private void btListen_Click(object sender, RoutedEventArgs e)
        {
            Int32 Res = wclIrDAServer.Listen();
            if (Res != wclErrors.WCL_E_SUCCESS)
                MessageBox.Show("Error: 0x" + Res.ToString("X8"));
        }

        private void btClose_Click(object sender, RoutedEventArgs e)
        {
            Int32 Res = wclIrDAServer.Close();
            if (Res != wclErrors.WCL_E_SUCCESS)
                MessageBox.Show("Error: 0x" + Res.ToString("X8"));
        }

        private void btClear_Click(object sender, RoutedEventArgs e)
        {
            lbEvents.Items.Clear();
        }

        private void btDisconnect_Click(object sender, RoutedEventArgs e)
        {
            if (lvClients.SelectedItems.Count == 0)
                MessageBox.Show("Select client");
            else
            {
                Int32 Res = (((DeviceItem)lvClients.SelectedItems[0]).Client).Disconnect();
                if (Res != wclErrors.WCL_E_SUCCESS)
                    MessageBox.Show("Error: 0x" + Res.ToString("X8"));
            }
        }

        private void btSend_Click(object sender, RoutedEventArgs e)
        {
            if (lvClients.SelectedItems.Count == 0)
                MessageBox.Show("Select client");
            else
            {
                Byte[] Ansi = Encoding.ASCII.GetBytes(edText.Text);
                UInt32 Sent = 0;
                Int32 Res = (((DeviceItem)lvClients.SelectedItems[0]).Client).Write(Ansi,
                    out Sent);
                if (Res != wclErrors.WCL_E_SUCCESS)
                    MessageBox.Show("Error: 0x" + Res.ToString("X8"));
                else
                    MessageBox.Show("Sent: " + Sent.ToString() + " from " + Ansi.Length.ToString());
            }
        }

        private void fmMain_Closed(object sender, EventArgs e)
        {
            wclIrDAServer.Close();
            wclIrDAServer = null;
        }

        private void btGet_Click(object sender, RoutedEventArgs e)
        {
            if (lvClients.SelectedItems.Count == 0)
                MessageBox.Show("Select client");
            else
            {
                wclIrDAServerClientConnection Client = ((DeviceItem)lvClients.SelectedItems[0]).Client;

                // Get read buffer.
                UInt32 Size = 0;
                Int32 Res = Client.GetReadBufferSize(out Size);
                if (Res != wclErrors.WCL_E_SUCCESS)
                    lbEvents.Items.Add("Get read buffer error: 0x" + Res.ToString("X8"));
                else
                    edReadBuffer.Text = Size.ToString();

                // Get write buffer.
                Size = 0;
                Res = Client.GetWriteBufferSize(out Size);
                if (Res != wclErrors.WCL_E_SUCCESS)
                    lbEvents.Items.Add("Get write buffer error: 0x" + Res.ToString("X8"));
                else
                    edWriteBuffer.Text = Size.ToString();
            }
        }

        private void btSet_Click(object sender, RoutedEventArgs e)
        {
            if (lvClients.SelectedItems.Count == 0)
                MessageBox.Show("Select client");
            else
            {
                wclIrDAServerClientConnection Client = ((DeviceItem)lvClients.SelectedItems[0]).Client;

                // Set read buffer.
                UInt32 Size = Convert.ToUInt32(edReadBuffer.Text);
                Int32 Res = Client.SetReadBufferSize(Size);
                if (Res != wclErrors.WCL_E_SUCCESS)
                    lbEvents.Items.Add("Set read buffer error: 0x" + Res.ToString("X8"));

                // Set write buffer.
                Size = Convert.ToUInt32(edWriteBuffer.Text);
                Res = Client.SetWriteBufferSize(Size);
                if (Res != wclErrors.WCL_E_SUCCESS)
                    lbEvents.Items.Add("Set write buffer error: 0x" + Res.ToString("X8"));
            }
        }

        private void btAddRecord_Click(object sender, RoutedEventArgs e)
        {
            switch (cbIasValueType.SelectedIndex)
            {
                case 0:
                    {
                        int Res = wclIrDAServer.AddIasRecord("IrDAFramework", "Integer value",
                            1234567);
                        if (Res != wclErrors.WCL_E_SUCCESS)
                            lbEvents.Items.Add("Add IAS record error: 0x" + Res.ToString("X8"));
                    }
                    break;

                case 1:
                    {
                        Byte[] Arr = { 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08 };
                        int Res = wclIrDAServer.AddIasRecord("IrDAFramework", "Octet Sequence", Arr);
                        if (Res != wclErrors.WCL_E_SUCCESS)
                            lbEvents.Items.Add("Add IAS record error: 0x" + Res.ToString("X8"));
                    }
                    break;

                case 2:
                    {
                        Byte[] Arr = { 85, 115, 101, 114, 32, 115, 116, 114, 105, 110, 103 };
                        int Res = wclIrDAServer.AddIasRecord("IrDAFramework", "User string",
                            wclIrDACharSet.csAscii, Arr);
                        if (Res != wclErrors.WCL_E_SUCCESS)
                            lbEvents.Items.Add("Add IAS record error: 0x" + Res.ToString("X8"));
                    }
                    break;
            }
        }
    }
}
