﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.IO;
using Microsoft.Win32;

using wclCommon;
using wclCommunication;
using wclIrDA;

namespace OppClient
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private wclIrDAReceiver wclIrDAReceiver;
        private wclIrDAClient wclIrDAClient;
        private wclObexOppClient FOpp;

        internal class DeviceItem
        {
            public String Address { get; set; }
            public String Name { get; set; }
        };

        public MainWindow()
        {
            InitializeComponent();
        }

        private void fmMain_Loaded(object sender, RoutedEventArgs e)
        {
            wclIrDAReceiver = new wclIrDAReceiver();

            wclIrDAClient = new wclIrDAClient();
            wclIrDAClient.OnDisconnect += new wclClientConnectionDisconnectEvent(wclIrDAClient_OnDisconnect);
            wclIrDAClient.OnConnect += new wclClientConnectionConnectEvent(wclIrDAClient_OnConnect);
            wclIrDAClient.OnDestroyProcessor += new wclClientConnectionProcessorEvent(wclIrDAClient_OnDestroyProcessor);
            wclIrDAClient.OnCreateProcessor += new wclClientConnectionProcessorEvent(wclIrDAClient_OnCreateProcessor);

            // We use ObjectPush Profile so must set the profile's UUID.
            wclIrDAClient.Service = "OBEX";
            wclIrDAClient.Mode = wclIrDAConnectionMode.cmIrComm3Wire; // We ned this for OBEX communication.

            FOpp = null;
        }

        void wclIrDAClient_OnCreateProcessor(object Sender, wclClientDataConnection Connection)
        {
            // here we must create the data processor for the connection. In this demo
            // we use OPPClient.
            FOpp = new wclObexOppClient(Connection);
            FOpp.OnConnect += new wclObexOperationResultEvent(FOpp_OnConnect);
            FOpp.OnDisconnect += new wclObexOperationResultEvent(FOpp_OnDisconnect);
            FOpp.OnGetComplete += new wclObexObjectOperationResultEvent(FOpp_OnGetComplete);
            FOpp.OnPutComplete += new wclObexObjectOperationResultEvent(FOpp_OnPutComplete);
            FOpp.OnProgress += new wclObexOperationProgressEvent(FOpp_OnProgress);
        }

        void FOpp_OnDisconnect(object Sender, int Error, string Description)
        {
            lbLog.Items.Add("OBEX session closed with result: 0x" + Error.ToString("X8") +
                ". Description: " + Description);
        }

        void FOpp_OnConnect(object Sender, int Error, string Description)
        {
            lbLog.Items.Add("OBEX session opened with result: 0x" + Error.ToString("X8") +
                ". Description: " + Description);
        }

        void FOpp_OnPutComplete(object Sender, int Error, string Description, System.IO.Stream Stream)
        {
            lbLog.Items.Add("File sending completed with result: 0x" + Error.ToString("X8") +
                ". Description: " + Description);

            ProgressBar.Value = 0;
            ProgressBar.Maximum = 0;

            // We must destroy the stream object here.
            Stream.Close();
            Stream.Dispose();
        }

        void FOpp_OnProgress(object Sender, uint Size, uint Position)
        {
            ProgressBar.Maximum = (int)Size;
            ProgressBar.Value = (int)Position;
        }

        void FOpp_OnGetComplete(object Sender, int Error, string Description, Stream Stream)
        {
            lbLog.Items.Add("File receiving completed with result: 0x" +
                Error.ToString("X8") + ". Description: " + Description);

            ProgressBar.Value = 0;
            ProgressBar.Maximum = 0;

            // We must destroy the stream object here.
            Stream.Close();
            Stream.Dispose();
        }

        void wclIrDAClient_OnDestroyProcessor(object Sender, wclClientDataConnection Connection)
        {
            // Do we have any data processor created?
            if (FOpp != null)
                // Make sure it is our connection is goind to destroy.
                if (Connection.Processor == FOpp)
                {
                    // Ok, destroy the data processor here.
                    FOpp.Dispose();
                    FOpp = null;
                }
        }

        void wclIrDAClient_OnConnect(object Sender, int Error)
        {
            lbLog.Items.Add("Connect. Operation result: 0x" + Error.ToString("X8"));
        }

        void wclIrDAClient_OnDisconnect(object Sender, int Reason)
        {
            lbLog.Items.Add("Disconnected with reason: 0x" + Reason.ToString("X8"));
        }

        private void btClear_Click(object sender, RoutedEventArgs e)
        {
            lbLog.Items.Clear();
        }

        private void fmMain_Closed(object sender, EventArgs e)
        {
            wclIrDAClient.Disconnect();
        }

        private void btDiscover_Click(object sender, RoutedEventArgs e)
        {
            lvDevices.Items.Clear();

            wclIrDADevice[] Devices = null;
            Int32 Res = wclIrDAReceiver.Discover(out Devices);
            if (Res != wclErrors.WCL_E_SUCCESS)
                MessageBox.Show("Failed to start discovering: 0x" + Res.ToString("X8"));
            else
            {
                if (Devices != null)
                {
                    foreach (wclIrDADevice Device in Devices)
                    {
                        lvDevices.Items.Add(new DeviceItem
                        {
                            Address = Device.Address.ToString("X8"),
                            Name = Device.Name
                        });
                    }
                }
            }
        }

        private void btDisconnect_Click(object sender, RoutedEventArgs e)
        {
            Int32 Res = wclIrDAClient.Disconnect();
            if (Res != wclErrors.WCL_E_SUCCESS)
                MessageBox.Show("Disconnect failed: 0x" + Res.ToString("X8"));
        }

        private void btCloseSession_Click(object sender, RoutedEventArgs e)
        {
            if (FOpp == null)
                MessageBox.Show("Not connected");
            else
            {
                Int32 Res = FOpp.Disconnect("");
                if (Res != wclErrors.WCL_E_SUCCESS)
                    MessageBox.Show("Close session failed: 0x" + Res.ToString("X8"));
            }
        }

        private void btOpenSession_Click(object sender, RoutedEventArgs e)
        {
            if (FOpp == null)
                MessageBox.Show("Not connected");
            else
            {
                Int32 Res = FOpp.Connect();
                if (Res != wclErrors.WCL_E_SUCCESS)
                    MessageBox.Show("Open session failed: 0x" + Res.ToString("X8"));
            }
        }

        private void btConnect_Click(object sender, RoutedEventArgs e)
        {
            if (wclIrDAClient.State != wclClientState.csDisconnected)
                MessageBox.Show("Client is connected");
            else
            {
                if (lvDevices.SelectedItems.Count == 0)
                    MessageBox.Show("Select device");
                else
                {
                    UInt32 Addr = Convert.ToUInt32(((DeviceItem)lvDevices.SelectedItems[0]).Address, 16);
                    wclIrDAClient.Address = Addr;
                    Int32 Res = wclIrDAClient.Connect();
                    if (Res != wclErrors.WCL_E_SUCCESS)
                        MessageBox.Show("Failed to connect: 0x" + Res.ToString("X8"));
                }
            }
        }

        private void btAbort_Click(object sender, RoutedEventArgs e)
        {
            if (FOpp == null)
                MessageBox.Show("Not connected");
            else
            {
                Int32 Res = FOpp.Abort("");
                if (Res != wclErrors.WCL_E_SUCCESS)
                    MessageBox.Show("Abort failed: 0x" + Res.ToString("X8"));
            }
        }

        private void btSelect_Click(object sender, RoutedEventArgs e)
        {
            OpenFileDialog openFileDialog = new OpenFileDialog();
            if (openFileDialog.ShowDialog() == true)
                edFileName.Text = openFileDialog.FileName;
        }

        private void btSend_Click(object sender, RoutedEventArgs e)
        {
            if (FOpp == null)
                MessageBox.Show("Not connected");
            else
                if (edFileName.Text == "")
                    MessageBox.Show("Select file");
                else
                {
                    FileStream Stream = new FileStream(edFileName.Text, FileMode.Open);
                    Int32 Res = FOpp.Put(System.IO.Path.GetFileName(edFileName.Text), "", Stream);
                    if (Res != wclErrors.WCL_E_SUCCESS)
                    {
                        MessageBox.Show("Send file failed: 0x" + Res.ToString("X8"));
                        Stream.Close();
                        Stream.Dispose();
                    }
                }
        }

        private void btGetvCard_Click(object sender, RoutedEventArgs e)
        {
            if (FOpp == null)
                MessageBox.Show("Not connected");
            else
            {
                FileStream Stream = new FileStream("vcard.vcf", FileMode.CreateNew);
                // Request default vCard object.
                Int32 Res = FOpp.Get("text/x-vCard", Stream);
                if (Res != wclErrors.WCL_E_SUCCESS)
                {
                    MessageBox.Show("GetvCard failed: 0x" + Res.ToString("X8"));
                    Stream.Close();
                    Stream.Dispose();
                }
            }
        }

        private void btGetCaps_Click(object sender, RoutedEventArgs e)
        {
            if (FOpp == null)
                MessageBox.Show("Not connected");
            else
            {
                FileStream Stream = new FileStream("caps.xml", FileMode.CreateNew);
                // Request Device Capability object.
                Int32 Res = FOpp.Get("x-obex/capability", Stream);
                if (Res != wclErrors.WCL_E_SUCCESS)
                {
                    MessageBox.Show("GetCaps failed: 0x" + Res.ToString("X8"));
                    Stream.Close();
                    Stream.Dispose();
                }
            }
        }
    }
}
