using System;
using System.Text;
using System.Windows.Forms;

using wclIrDA;
using wclCommunication;
using wclCommon;

namespace IrDAClient
{
    public partial class fmMain : Form
    {
        private wclIrDAClient wclIrDAClient;
        private wclIrDAReceiver wclIrDAReceiver;

        public fmMain()
        {
            InitializeComponent();
        }

        private void fmMain_Load(object sender, EventArgs e)
        {
            wclIrDAClient = new wclIrDAClient();
            wclIrDAClient.OnConnect += new wclClientConnectionConnectEvent(wclIrDAClient_OnConnect);
            wclIrDAClient.OnData += new wclConnectionDataEvent(wclIrDAClient_OnData);
            wclIrDAClient.OnDisconnect += new wclClientConnectionDisconnectEvent(wclIrDAClient_OnDisconnect);

            wclIrDAReceiver = new wclIrDAReceiver();
        }

        void wclIrDAClient_OnDisconnect(object Sender, int Reason)
        {
            lbEvents.Items.Add("Disconnected: 0x" + Reason.ToString("X8"));
            
            edReadBuffer.Text = "0";
            edWriteBuffer.Text = "0";
        }

        void wclIrDAClient_OnData(object Sender, byte[] Data)
        {
            if (Data != null && Data.Length > 0)
            {
                String Str = Encoding.ASCII.GetString(Data);

                lbEvents.Items.Add("Received: " + Str);
            }
        }

        void wclIrDAClient_OnConnect(object Sender, int Error)
        {
            if (Error == wclErrors.WCL_E_SUCCESS)
            {
                lbEvents.Items.Add("Connected");
                
                // Try to get read and write buffers size.
                UInt32 Size = 0;
                Int32 Res = wclIrDAClient.GetReadBufferSize(out Size);
                if (Res != wclErrors.WCL_E_SUCCESS)
                    lbEvents.Items.Add("Get read buffer error: 0x" + Res.ToString("X8"));
                else
                    edReadBuffer.Text = Size.ToString();
                
                Size = 0;
                Res = wclIrDAClient.GetWriteBufferSize(out Size);
                if (Res != wclErrors.WCL_E_SUCCESS)
                    lbEvents.Items.Add("Get write buffer error: 0x" + Res.ToString("X8"));
                else
                    edWriteBuffer.Text = Size.ToString();
            }
            else
                lbEvents.Items.Add("Connect error: 0x" + Error.ToString("X8"));
        }

        private void btDiscover_Click(object sender, EventArgs e)
        {
            lvDevices.Items.Clear();
            
            wclIrDADevice[] Devices = null;
            Int32 Res = wclIrDAReceiver.Discover(out Devices);
            try
            {
                if (Res != wclErrors.WCL_E_SUCCESS)
                    MessageBox.Show("Error: 0x" + Res.ToString("X8"));
                else
                    if (Devices == null || Devices.Length == 0)
                        MessageBox.Show("No devices were found.");
                    else
                        for (int i = 0; i < Devices.Length; i++)
                        {
                            ListViewItem Item = lvDevices.Items.Add(Devices[i].Address.ToString("X8"));
                            Item.SubItems.Add(Devices[i].Name);
                        }
            }
            finally
            {
                Devices = null;
            }
        }

        private void btClear_Click(object sender, EventArgs e)
        {
            lbEvents.Items.Clear();
        }

        private void btDisconnect_Click(object sender, EventArgs e)
        {
            Int32 Res = wclIrDAClient.Disconnect();
            if (Res != wclErrors.WCL_E_SUCCESS)
                MessageBox.Show("Error: 0x" + Res.ToString("X8"));
        }

        private void btConnect_Click(object sender, EventArgs e)
        {
            if (lvDevices.SelectedItems.Count == 0)
                MessageBox.Show("Select device");
            else
            {
                wclIrDAClient.Address = Convert.ToUInt32(lvDevices.SelectedItems[0].Text, 16);
                wclIrDAClient.Timeout = Convert.ToUInt32(edTimeout.Text);

                Int32 Res = wclIrDAClient.Connect();
                if (Res != wclErrors.WCL_E_SUCCESS)
                    MessageBox.Show("Error: 0x" + Res.ToString("X8"));
            }
        }

        private void edSend_Click(object sender, EventArgs e)
        {
            Byte[] Ansi = Encoding.ASCII.GetBytes(edText.Text);
            UInt32 Sent = 0;
            Int32 Res = wclIrDAClient.Write(Ansi, out Sent);
            if (Res != wclErrors.WCL_E_SUCCESS)
                MessageBox.Show("Error: 0x" + Res.ToString("X8"));
            else
                MessageBox.Show("Sent: " + Sent.ToString() + " from " + Ansi.Length.ToString());
        }

        private void fmMain_FormClosed(object sender, FormClosedEventArgs e)
        {
            wclIrDAClient.Disconnect();
            wclIrDAClient = null;
            wclIrDAReceiver = null;
        }

        private void btSetBuffers_Click(object sender, EventArgs e)
        {
            // Try to set read buffer size first.
            UInt32 Size = Convert.ToUInt32(edReadBuffer.Text);
            Int32 Res = wclIrDAClient.SetReadBufferSize(Size);
            if (Res == wclErrors.WCL_E_SUCCESS)
                lbEvents.Items.Add("Read buffer size changed.");
            else
                lbEvents.Items.Add("Set read buffer size error: 0x" + Res.ToString("X8"));
            
            // Now write buffer.
            Size = Convert.ToUInt32(edWriteBuffer.Text);
            Res = wclIrDAClient.SetWriteBufferSize(Size);
            if (Res == wclErrors.WCL_E_SUCCESS)
                lbEvents.Items.Add("Write buffer size changed.");
            else
                lbEvents.Items.Add("Set write buffer size error: 0x" + Res.ToString("X8"));
        }
    }
}