Public Class fmMain

    Private WithEvents wclIrDAClient As wclIrDAClient
    Private wclIrDAReceiver As wclIrDAReceiver

    Private Sub fmMain_FormClosed(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosedEventArgs) Handles Me.FormClosed
        wclIrDAClient.Disconnect()
        wclIrDAClient = Nothing
        wclIrDAReceiver = Nothing
    End Sub

    Private Sub fmMain_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        wclIrDAClient = New wclIrDAClient
        wclIrDAReceiver = New wclIrDAReceiver
    End Sub

    Private Sub wclIrDAClient_OnConnect(ByVal Sender As Object, ByVal [Error] As Integer) Handles wclIrDAClient.OnConnect
        If [Error] = wclErrors.WCL_E_SUCCESS Then
            lbEvents.Items.Add("Connected")
            ' Try to get read and write buffers size.
            Dim Size As UInt32 = 0
            Dim Res As Int32 = wclIrDAClient.GetReadBufferSize(Size)
            If Res <> wclErrors.WCL_E_SUCCESS Then
                lbEvents.Items.Add("Get read buffer error: 0x" + Res.ToString("X8"))
            Else
                edReadBuffer.Text = Size.ToString()

                Size = 0
                Res = wclIrDAClient.GetWriteBufferSize(Size)
                If Res <> wclErrors.WCL_E_SUCCESS Then
                    lbEvents.Items.Add("Get write buffer error: 0x" + Res.ToString("X8"))
                Else
                    edWriteBuffer.Text = Size.ToString()
                End If
            End If
        Else
            lbEvents.Items.Add("Connect error: 0x" + [Error].ToString("X8"))
        End If
    End Sub

    Private Sub wclIrDAClient_OnData(ByVal Sender As Object, ByVal Data() As Byte) Handles wclIrDAClient.OnData
        If Data IsNot Nothing AndAlso Data.Length > 0 Then
            Dim Str As String = System.Text.Encoding.ASCII.GetString(Data)

            lbEvents.Items.Add("Received: " + Str)
        End If
    End Sub

    Private Sub wclIrDAClient_OnDisconnect(ByVal Sender As Object, ByVal Reason As Integer) Handles wclIrDAClient.OnDisconnect
        lbEvents.Items.Add("Disconnected: " + Reason.ToString())

        edReadBuffer.Text = "0"
        edWriteBuffer.Text = "0"
    End Sub

    Private Sub btDiscover_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btDiscover.Click
        lvDevices.Items.Clear()

        Dim Devices() As wclIrDADevice = Nothing
        Dim Res As Int32 = wclIrDAReceiver.Discover(Devices)
        Try
            If Res <> wclErrors.WCL_E_SUCCESS Then
                MessageBox.Show("Error: 0x" + Res.ToString("X8"))
            Else
                If Devices Is Nothing OrElse Devices.Length = 0 Then
                    MessageBox.Show("No devices were found.")
                Else
                    Dim i As Integer
                    For i = 0 To Devices.Length - 1
                        Dim Item As ListViewItem = lvDevices.Items.Add(Devices(i).Address.ToString("X8"))
                        Item.SubItems.Add(Devices(i).Name)
                    Next i
                End If
            End If
        Finally
            Devices = Nothing
        End Try
    End Sub

    Private Sub btClear_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btClear.Click
        lbEvents.Items.Clear()
    End Sub

    Private Sub btDisconnect_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btDisconnect.Click
        Dim Res As Int32 = wclIrDAClient.Disconnect()
        If Res <> wclErrors.WCL_E_SUCCESS Then
            MessageBox.Show("Error: 0x" + Res.ToString("X8"))
        End If
    End Sub

    Private Sub btConnect_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btConnect.Click
        If lvDevices.SelectedItems.Count = 0 Then
            MessageBox.Show("Select device")
        Else
            wclIrDAClient.Address = Convert.ToUInt32(lvDevices.SelectedItems(0).Text, 16)
            wclIrDAClient.Timeout = Convert.ToUInt32(edTimeout.Text)

            Dim Res As Int32 = wclIrDAClient.Connect()
            If Res <> wclErrors.WCL_E_SUCCESS Then
                MessageBox.Show("Error: 0x" + Res.ToString("X8"))
            End If
        End If
    End Sub

    Private Sub edSend_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles edSend.Click
        Dim Ansi() As Byte = System.Text.Encoding.ASCII.GetBytes(edText.Text)
        Dim Sent As UInt32 = 0
        Dim Res As Int32 = wclIrDAClient.Write(Ansi, Sent)
        If Res <> wclErrors.WCL_E_SUCCESS Then
            MessageBox.Show("Error: 0x" + Res.ToString("X8"))
        Else
            MessageBox.Show("Sent: " + Sent.ToString() + " from " + Ansi.Length.ToString())
        End If
    End Sub

    Private Sub btSetBuffers_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btSetBuffers.Click
        ' Try to set read buffer size first.
        Dim Size As UInt32 = Convert.ToUInt32(edReadBuffer.Text)
        Dim Res As Int32 = wclIrDAClient.SetReadBufferSize(Size)
        If Res = wclErrors.WCL_E_SUCCESS Then
            lbEvents.Items.Add("Read buffer size changed.")
        Else
            lbEvents.Items.Add("Set read buffer size error: 0x" + Res.ToString("X8"))
        End If

        ' Now write buffer.
        Size = Convert.ToUInt32(edWriteBuffer.Text)
        Res = wclIrDAClient.SetWriteBufferSize(Size)
        If Res = wclErrors.WCL_E_SUCCESS Then
            lbEvents.Items.Add("Write buffer size changed.")
        Else
            lbEvents.Items.Add("Set write buffer size error: 0x" + Res.ToString("X8"))
        End If
    End Sub
End Class
