Public Class fmMain
    Private WithEvents wclIrDAReceiver As wclIrDAReceiver
    Private WithEvents FPowerMonitor As wclPowerEventsMonitor

    Private Sub fmMain_FormClosed(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosedEventArgs) Handles Me.FormClosed
        wclIrDAReceiver.StopLazyDiscovering()
        wclIrDAReceiver.Unsubscribe()
        wclIrDAReceiver = Nothing

        FPowerMonitor.Close()
        FPowerMonitor = Nothing
    End Sub

    Private Sub fmMain_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        wclIrDAReceiver = New wclIrDAReceiver()

        cbClassName.SelectedIndex = 0
        cbClassName_SelectedIndexChanged(cbClassName, EventArgs.Empty)

        FPowerMonitor = New wclPowerEventsMonitor()
        FPowerMonitor.Open()
    End Sub

    Private Sub wclIrDAReceiver_OnChange(ByVal sender As Object, ByVal e As System.EventArgs) Handles wclIrDAReceiver.OnChange
        If (wclIrDAReceiver.Available) Then
            lbEvents.Items.Add("Hardware changed. IrDA available.")
        Else
            lbEvents.Items.Add("Hardware changed. IrDA NOT available.")
        End If
    End Sub

    Private Sub wclIrDAReceiver_OnDevicesFound(ByVal Sender As Object, ByVal Device As wclIrDADevice) Handles wclIrDAReceiver.OnDevicesFound
        lbEvents.Items.Add("Device found: " + Device.Address.ToString("X8") + "; " + Device.Name)
    End Sub

    Private Sub cbClassName_SelectedIndexChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles cbClassName.SelectedIndexChanged
        cbAttributeName.Items.Clear()

        Select Case cbClassName.SelectedIndex
            Case 0
                cbAttributeName.Items.Add("DeviceName")
                cbAttributeName.Items.Add("IrLMPSupport")

            Case 1
                cbAttributeName.Items.Add("Parameters")
                cbAttributeName.Items.Add("IrDA:IrLMP:LsapSel")
                cbAttributeName.Items.Add("IrDA:TinyTP:LsapSel")
                cbAttributeName.Items.Add("IrDA:InstanceName")

            Case 2
                cbAttributeName.Items.Add("IrDA:TinyTP:LsapSel")

            Case 3
                cbAttributeName.Items.Add("IrDA:IrLMP:LsapSel")

            Case 4
                cbAttributeName.Items.Add("Integer value")
                cbAttributeName.Items.Add("Octet Sequence")
                cbAttributeName.Items.Add("User string")

        End Select

        cbAttributeName.SelectedIndex = 0
    End Sub

    Private Sub btDiscover_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btDiscover.Click
        lvDevices.Items.Clear()

        Dim Devices() As wclIrDADevice = Nothing
        Dim Res As Int32 = wclIrDAReceiver.Discover(Devices)
        If Res <> wclErrors.WCL_E_SUCCESS Then
            MessageBox.Show("Error: 0x" + Res.ToString("X8"))
        Else
            If Devices Is Nothing OrElse Devices.Length = 0 Then
                MessageBox.Show("Nothing found")
            Else
                Dim i As Integer
                For i = 0 To Devices.Length - 1
                    Dim Device As wclIrDADevice = Devices(i)

                    Dim Item As ListViewItem = lvDevices.Items.Add(Device.Address.ToString("X8"))
                    Item.SubItems.Add(Device.Name)
                    Item.SubItems.Add(Device.Hints.ToString())
                    Item.SubItems.Add(Device.CharSet.ToString())
                Next i
            End If
        End If
    End Sub

    Private Sub btSubscribe_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btSubscribe.Click
        Dim Res As Int32 = wclIrDAReceiver.Subscribe()
        If Res <> wclErrors.WCL_E_SUCCESS Then
            MessageBox.Show("Error: 0x" + Res.ToString("X8"))
        End If
    End Sub

    Private Sub btUnsubscribe_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btUnsubscribe.Click
        Dim Res As Int32 = wclIrDAReceiver.Unsubscribe()
        If Res <> wclErrors.WCL_E_SUCCESS Then
            MessageBox.Show("Error: 0x" + Res.ToString("X8"))
        End If
    End Sub

    Private Sub btClear_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btClear.Click
        lbEvents.Items.Clear()
    End Sub

    Private Sub btStartLazyDiscovering_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btStartLazyDiscovering.Click
        Dim Res As Int32 = wclIrDAReceiver.StartLazyDiscovering()
        If Res <> wclErrors.WCL_E_SUCCESS Then
            MessageBox.Show("Error: 0x" + Res.ToString("X8"))
        End If
    End Sub

    Private Sub btStopLazyDiscovering_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btStopLazyDiscovering.Click
        Dim Res As Int32 = wclIrDAReceiver.StopLazyDiscovering()
        If Res <> wclErrors.WCL_E_SUCCESS Then
            MessageBox.Show("Error: 0x" + Res.ToString("X8"))
        End If
    End Sub

    Private Sub btQuery_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btQuery.Click
        If lvDevices.SelectedItems.Count = 0 Then
            MessageBox.Show("Select device")
        Else
            Dim Attrib As wclIrDAAttribute = Nothing
            Dim Res As Int32 = wclIrDAReceiver.QueryAttribute(Convert.ToUInt32(lvDevices.SelectedItems(0).Text, 16), cbClassName.SelectedItem.ToString(), cbAttributeName.SelectedItem.ToString(), Attrib)
            If Res <> wclErrors.WCL_E_SUCCESS Then
                MessageBox.Show("Error: 0x" + Res.ToString("X8"))
            Else
                Select Case Attrib.ValueType
                    Case wclIrDAAttributeType.atInteger
                        MessageBox.Show("Integer: " + Attrib.IntegerValue.ToString())

                    Case wclIrDAAttributeType.atSequence
                        Dim Str As String = ""
                        If Attrib.SequenceValue IsNot Nothing Then
                            If Attrib.SequenceValue.Length > 0 Then
                                Dim i As Integer
                                For i = 0 To Attrib.SequenceValue.Length - 1
                                    Str = Str + Attrib.SequenceValue(i).ToString("X2")
                                Next i
                            End If

                            Attrib.SequenceValue = Nothing
                        End If
                        MessageBox.Show("Sequence: " + Str)

                    Case wclIrDAAttributeType.atString
                        MessageBox.Show("String: " + Attrib.StringValue)
                End Select
            End If
        End If
    End Sub

    Private Sub FPowerMonitor_OnPowerStateChanged(Sender As Object, State As wclCommon.wclPowerState) Handles FPowerMonitor.OnPowerStateChanged
        Select State
            Case wclPowerState.psResumeAutomatic
                lbEvents.Items.Add("Power state: psResumeAutomatic")

            Case wclPowerState.psResume
                lbEvents.Items.Add("Power state: psResume")

            Case wclPowerState.psSuspend
                wclIrDAReceiver.StopLazyDiscovering()
                wclIrDAReceiver.Unsubscribe()

                lbEvents.Items.Add("Power state: psSuspend")

            Case wclPowerState.psUnknown
                lbEvents.Items.Add("Power state: psUnknown")
        End Select
    End Sub
End Class
