Public Class fmMain
    Private wclIrDAReceiver As wclIrDAReceiver
    Private WithEvents wclIrDAClient As wclIrDAClient
    Private WithEvents FOpp As wclObexOppClient

    Private Sub fmMain_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        wclIrDAReceiver = New wclIrDAReceiver()
        wclIrDAClient = New wclIrDAClient()

        ' We use ObjectPush Profile so must set the profile's UUID.
        wclIrDAClient.Service = "OBEX"
        wclIrDAClient.Mode = wclIrDAConnectionMode.cmIrComm3Wire ' We need this for OBEX

        FOpp = Nothing
    End Sub

    Private Sub FOpp_OnConnect(ByVal Sender As Object, ByVal [Error] As Integer, ByVal Description As String) Handles FOpp.OnConnect
        lbLog.Items.Add("OBEX session opened with result: 0x" + [Error].ToString("X8") + ". Description: " + Description)
    End Sub

    Private Sub FOpp_OnDisconnect(ByVal Sender As Object, ByVal [Error] As Integer, ByVal Description As String) Handles FOpp.OnDisconnect
        lbLog.Items.Add("OBEX session closed with result: 0x" + [Error].ToString("X8") + ". Description: " + Description)
    End Sub

    Private Sub FOpp_OnGetComplete(ByVal Sender As Object, ByVal [Error] As Integer, ByVal Description As String, ByVal Stream As System.IO.Stream) Handles FOpp.OnGetComplete
        lbLog.Items.Add("File receiving completed with result: 0x" + [Error].ToString("X8") + ". Description: " + Description)

        ProgressBar.Value = 0
        ProgressBar.Maximum = 0

        ' We must destroy the stream object here.
        Stream.Close()
        Stream.Dispose()
    End Sub

    Private Sub FOpp_OnProgress(ByVal Sender As Object, ByVal Size As UInteger, ByVal Position As UInteger) Handles FOpp.OnProgress
        ProgressBar.Maximum = Size
        ProgressBar.Value = Position
    End Sub

    Private Sub FOpp_OnPutComplete(ByVal Sender As Object, ByVal [Error] As Integer, ByVal Description As String, ByVal Stream As System.IO.Stream) Handles FOpp.OnPutComplete
        lbLog.Items.Add("File sending completed with result: 0x" + [Error].ToString("X8") + ". Description: " + Description)

        ProgressBar.Value = 0
        ProgressBar.Maximum = 0

        ' We must destroy the stream object here.
        Stream.Close()
        Stream.Dispose()
    End Sub

    Private Sub btClear_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btClear.Click
        lbLog.Items.Clear()
    End Sub

    Private Sub fmMain_FormClosed(ByVal sender As System.Object, ByVal e As System.Windows.Forms.FormClosedEventArgs) Handles MyBase.FormClosed
        wclIrDAClient.Disconnect()
    End Sub

    Private Sub btDiscover_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btDiscover.Click
        lvDevices.Items.Clear()

        Dim Devices As wclIrDADevice() = Nothing
        ' Discover classic devices only.
        Dim Res As Int32 = wclIrDAReceiver.Discover(Devices)
        If Res <> wclErrors.WCL_E_SUCCESS Then
            MessageBox.Show("Failed to start discovering: 0x" + Res.ToString("X8"))
        Else
            If Not Devices Is Nothing Then
                For Each Device As wclIrDADevice In Devices
                    Dim Item As ListViewItem = lvDevices.Items.Add(Device.Address.ToString("X8"))
                    Item.SubItems.Add(Device.Name)
                Next Device
            End If
        End If
    End Sub

    Private Sub btDisconnect_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btDisconnect.Click
        Dim Res As Int32 = wclIrDAClient.Disconnect()
        If Res <> wclErrors.WCL_E_SUCCESS Then
            MessageBox.Show("Disconnect failed: 0x" + Res.ToString("X8"))
        End If
    End Sub

    Private Sub btCloseSession_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btCloseSession.Click
        If FOpp Is Nothing Then
            MessageBox.Show("Not connected")
        Else
            Dim Res As Int32 = FOpp.Disconnect("")
            If Res <> wclErrors.WCL_E_SUCCESS Then
                MessageBox.Show("Close session failed: 0x" + Res.ToString("X8"))
            End If
        End If
    End Sub

    Private Sub btOpenSession_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btOpenSession.Click
        If FOpp Is Nothing Then
            MessageBox.Show("Not connected")
        Else
            Dim Res As Int32 = FOpp.Connect()
            If Res <> wclErrors.WCL_E_SUCCESS Then
                MessageBox.Show("Open session failed: 0x" + Res.ToString("X8"))
            End If
        End If
    End Sub

    Private Sub btConnect_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btConnect.Click
        If wclIrDAClient.State <> wclClientState.csDisconnected Then
            MessageBox.Show("Client is connected")
        Else
            If lvDevices.SelectedItems.Count = 0 Then
                MessageBox.Show("Select device")
            Else
                Dim Addr As UInt32 = Convert.ToUInt32(lvDevices.SelectedItems(0).Text, 16)
                wclIrDAClient.Address = Addr
                Dim Res As Int32 = wclIrDAClient.Connect()
                If Res <> wclErrors.WCL_E_SUCCESS Then
                    MessageBox.Show("Failed to connect: 0x" + Res.ToString("X8"))
                End If
            End If
        End If
    End Sub

    Private Sub btAbort_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btAbort.Click
        If FOpp Is Nothing Then
            MessageBox.Show("Not connected")
        Else
            Dim Res As Int32 = FOpp.Abort("")
            If Res <> wclErrors.WCL_E_SUCCESS Then
                MessageBox.Show("Abort failed: 0x" + Res.ToString("X8"))
            End If
        End If
    End Sub

    Private Sub btSendFile_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btSendFile.Click
        If FOpp Is Nothing Then
            MessageBox.Show("Not connected")
        Else
            If edFileName.Text = "" Then
                MessageBox.Show("Select file")
            Else
                Dim Stream As FileStream = New FileStream(edFileName.Text, FileMode.Open)
                Dim Res As Int32 = FOpp.Put(Path.GetFileName(edFileName.Text), "", Stream)
                If Res <> wclErrors.WCL_E_SUCCESS Then
                    MessageBox.Show("Send file failed: 0x" + Res.ToString("X8"))
                    Stream.Close()
                    Stream.Dispose()
                End If
            End If
        End If
    End Sub

    Private Sub btGetVCard_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btGetVCard.Click
        If FOpp Is Nothing Then
            MessageBox.Show("Not connected")
        Else
            Dim Stream As FileStream = New FileStream("vcard.vcf", FileMode.CreateNew)
            ' Request default vCard object.
            Dim Res As Int32 = FOpp.Get("text/x-vCard", Stream)
            If Res <> wclErrors.WCL_E_SUCCESS Then
                MessageBox.Show("GetvCard failed: 0x" + Res.ToString("X8"))
                Stream.Close()
                Stream.Dispose()
            End If
        End If
    End Sub

    Private Sub btGetCaps_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btGetCaps.Click
        If FOpp Is Nothing Then
            MessageBox.Show("Not connected")
        Else
            Dim Stream As FileStream = New FileStream("caps.xml", FileMode.CreateNew)
            ' Request Device Capability object.
            Dim Res As Int32 = FOpp.Get("x-obex/capability", Stream)
            If Res <> wclErrors.WCL_E_SUCCESS Then
                MessageBox.Show("GetCaps failed: 0x" + Res.ToString("X8"))
                Stream.Close()
                Stream.Dispose()
            End If
        End If
    End Sub

    Private Sub btSelectFile_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btSelectFile.Click
        If openFileDialog.ShowDialog() = DialogResult.OK Then
            edFileName.Text = openFileDialog.FileName
        End If
    End Sub

    Private Sub wclIrDAClient_OnConnect(ByVal Sender As Object, ByVal [Error] As Integer) Handles wclIrDAClient.OnConnect
        lbLog.Items.Add("Connect. Operation result: 0x" + [Error].ToString("X8"))
    End Sub

    Private Sub wclIrDAClient_OnCreateProcessor(ByVal Sender As Object, ByVal Connection As wclCommunication.wclClientDataConnection) Handles wclIrDAClient.OnCreateProcessor
        ' here we must create the data processor for the connection. In this demo
        ' we use OPPClient.
        FOpp = New wclObexOppClient(Connection)
    End Sub

    Private Sub wclIrDAClient_OnDestroyProcessor(ByVal Sender As Object, ByVal Connection As wclCommunication.wclClientDataConnection) Handles wclIrDAClient.OnDestroyProcessor
        ' Do we have any data processor created?
        If Not FOpp Is Nothing Then
            ' Make sure it is our connection is goind to destroy.
            If Connection.Processor Is FOpp Then
                ' Ok, destroy the data processor here.
                FOpp.Dispose()
                FOpp = Nothing
            End If
        End If
    End Sub

    Private Sub wclIrDAClient_OnDisconnect(ByVal Sender As Object, ByVal Reason As Integer) Handles wclIrDAClient.OnDisconnect
        lbLog.Items.Add("Disconnected with reason: 0x" + Reason.ToString("X8"))
    End Sub
End Class
