Public Class fmMain
    Private WithEvents wclIrDAServer As wclIrDAServer
    Private FFileName As String

    Private Sub Trace(ByVal Msg As String)
        lbLog.Items.Add(Msg)
    End Sub

    Private Sub btClear_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btClear.Click
        lbLog.Items.Clear()
    End Sub

    Private Sub btListen_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btListen.Click
        Dim Res As Int32 = wclIrDAServer.Listen()
        If Res <> wclErrors.WCL_E_SUCCESS Then
            MessageBox.Show("Error: 0x" + Res.ToString("X8"))
        End If
    End Sub

    Private Sub btClose_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btClose.Click
        Dim Res As Int32 = wclIrDAServer.Close()
        If Res <> wclErrors.WCL_E_SUCCESS Then
            MessageBox.Show("Error: 0x" + Res.ToString("X8"))
        End If
    End Sub

    Private Sub fmMain_FormClosed(ByVal sender As System.Object, ByVal e As System.Windows.Forms.FormClosedEventArgs) Handles MyBase.FormClosed
        ' Simple try to close the server.
        wclIrDAServer.Close()
    End Sub

    Private Sub fmMain_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        wclIrDAServer = New wclIrDAServer()
        wclIrDAServer.Service = "OBEX"
        wclIrDAServer.Mode = wclIrDAConnectionMode.cmIrComm3Wire ' Required for OBEX connection
    End Sub

    Private Sub wclIrDAServer_OnClosed(ByVal Sender As Object, ByVal Reason As Integer) Handles wclIrDAServer.OnClosed
        Trace("Server closed. Reason:: 0x" + Reason.ToString("X8"))
    End Sub

    Private Sub wclIrDAServer_OnConnect(ByVal Sender As Object, ByVal Client As wclIrDA.wclIrDAServerClientConnection, ByVal [Error] As Integer) Handles wclIrDAServer.OnConnect
        If [Error] = wclErrors.WCL_E_SUCCESS Then
            Trace("Client " + Client.Address.ToString("X8") + " connected")
        Else
            Trace("Client " + Client.Address.ToString("X8") + " connect failed: 0x" + [Error].ToString("X8"))
        End If
    End Sub

    Private Sub wclIrDAServer_OnCreateProcessor(ByVal Sender As Object, ByVal Connection As wclCommunication.wclServerClientDataConnection) Handles wclIrDAServer.OnCreateProcessor
        ' Here we must create OBEX OPP server data processor.
        Dim Proc As wclObexOppServer = New wclObexOppServer(Connection)
        AddHandler Proc.OnConnect, AddressOf Proc_OnConnect
        AddHandler Proc.OnDisconnected, AddressOf Proc_OnDisconnected
        AddHandler Proc.OnPutBegin, AddressOf Proc_OnPutBegin
        AddHandler Proc.OnPutCompleted, AddressOf Proc_OnPutCompleted
        AddHandler Proc.OnPutProgress, AddressOf Proc_OnPutProgress
    End Sub

    Private Sub Proc_OnPutProgress(ByVal Sender As Object, ByVal Position As UInteger, ByVal Length As UInteger, ByRef [Continue] As Boolean)
        Trace("Sending file: " + Position.ToString() + " from " + Length.ToString())
        ' Do not foget to set Continue flag!
        [Continue] = True
    End Sub

    Private Sub Proc_OnPutCompleted(ByVal Sender As Object, ByVal [Error] As Integer, ByVal Stream As System.IO.Stream, ByRef Accept As Boolean)
        Dim Proc As wclObexOppServer = CType(Sender, wclObexOppServer)
        Dim Address As Int64 = CType(Proc.Connection, wclIrDAServerClientConnection).Address
        If [Error] = wclErrors.WCL_E_SUCCESS Then
            Dim Str As String = "Client " + Address.ToString("X8") + " completed sent file. Accept?"
            Accept = MessageBox.Show(Str, "Accept File", MessageBoxButtons.YesNo, MessageBoxIcon.Question) = DialogResult.Yes

            If Accept Then
                saveFileDialog.FileName = FFileName
                If saveFileDialog.ShowDialog() = DialogResult.OK Then
                    Stream.Seek(0, IO.SeekOrigin.Begin)
                    Dim fs As IO.FileStream = New IO.FileStream(saveFileDialog.FileName, IO.FileMode.CreateNew, IO.FileAccess.Write)
                    Dim Bytes(Stream.Length) As Byte
                    Stream.Read(Bytes, 0, Stream.Length)
                    fs.Write(Bytes, 0, Bytes.Length)
                    fs.Close()
                End If
            End If
        Else
            Trace("Send file error: 0x" + [Error].ToString("X8"))
            Accept = False
        End If
    End Sub

    Private Sub Proc_OnPutBegin(ByVal Sender As Object, ByVal Name As String, ByVal Description As String, ByVal Mime As String, ByVal Length As UInteger, ByRef Accept As Boolean)
        Dim Proc As wclObexOppServer = CType(Sender, wclObexOppServer)
        Dim Address As Int64 = CType(Proc.Connection, wclIrDAServerClientConnection).Address
        Dim Str As String = "Client " + Address.ToString("X8") + " send file " + Name + ". Accept?"
        Accept = MessageBox.Show(Str, "Accept file", MessageBoxButtons.YesNo, MessageBoxIcon.Question) = DialogResult.Yes

        If Accept Then
            FFileName = Name
        End If
    End Sub

    Private Sub Proc_OnDisconnected(ByVal Sender As Object, ByVal Reason As Integer, ByVal Description As String)
        Dim Proc As wclObexOppServer = CType(Sender, wclObexOppServer)
        Dim Address As Int64 = CType(Proc.Connection, wclIrDAServerClientConnection).Address

        Trace("Client " + Address.ToString("X8") + " closed OBEX session with reason: 0x" + Reason.ToString("X8") + ". Description: " + Description)
    End Sub

    Private Sub Proc_OnConnect(ByVal Sender As Object, ByVal Description As String)
        Dim Proc As wclObexOppServer = CType(Sender, wclObexOppServer)
        Dim Address As Int64 = CType(Proc.Connection, wclIrDAServerClientConnection).Address
        Dim Accept As Boolean = MessageBox.Show("Client " + Address.ToString("X8") + " requsts for connection: " + Description + ". Accept?", "Accept connection", MessageBoxButtons.YesNo, MessageBoxIcon.Question) = DialogResult.Yes

        Dim Res As Int32
        If Accept Then
            Res = Proc.Accept("Accepted by user")
        Else
            Res = Proc.Reject("Rejected by user")
        End If

        If Res <> wclErrors.WCL_E_SUCCESS Then
            MessageBox.Show("Operation failed: 0x" + Res.ToString("X8"))
            ' Disconnect client!
            Proc.Connection.Disconnect()
        End If
    End Sub

    Private Sub wclIrDAServer_OnDestroyProcessor(ByVal Sender As Object, ByVal Connection As wclCommunication.wclServerClientDataConnection) Handles wclIrDAServer.OnDestroyProcessor
        Trace("Data processor for client " + CType(Connection, wclIrDAServerClientConnection).Address.ToString("X8") + " is destroing")
        Connection.Processor.Dispose()
    End Sub

    Private Sub wclIrDAServer_OnDisconnect(ByVal Sender As Object, ByVal Client As wclIrDA.wclIrDAServerClientConnection, ByVal Reason As Integer) Handles wclIrDAServer.OnDisconnect
        Trace("Client " + Client.Address.ToString("X8") + " disconnected. Reason: 0x" + Reason.ToString("X8"))
    End Sub

    Private Sub wclIrDAServer_OnListen(ByVal sender As Object, ByVal e As System.EventArgs) Handles wclIrDAServer.OnListen
        Trace("Server listening")
    End Sub
End Class
